@@ -170,7 +170,7 @@ namespace dxvk {
     
     // Determine map mode based on our findings
     VkMemoryPropertyFlags memoryProperties = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
-    std::tie(m_mapMode, memoryProperties) = DetermineMapMode(&imageInfo);
+    std::tie(m_mapMode, memoryProperties) = DetermineMapMode(pDevice, &imageInfo);
     
     // If the image is mapped directly to host memory, we need
     // to enable linear tiling, and DXVK needs to be aware that
@@ -546,6 +546,7 @@ namespace dxvk {
 
   
   std::pair<D3D11_COMMON_TEXTURE_MAP_MODE, VkMemoryPropertyFlags> D3D11CommonTexture::DetermineMapMode(
+    const D3D11Device*          device,
     const DxvkImageCreateInfo*  pImageInfo) const {
     // Don't map an image unless the application requests it
     if (!m_desc.CPUAccessFlags)
@@ -563,8 +564,9 @@ namespace dxvk {
       return { D3D11_COMMON_TEXTURE_MAP_MODE_STAGING, 0u };
 
     // If the packed format and image format don't match, we need to use
-    // a staging buffer and perform format conversion when mapping.
-    if (m_packedFormat != pImageInfo->format)
+    // a staging buffer and perform format conversion when mapping. The
+    // same is true if the game is broken and requires tight packing.
+    if (m_packedFormat != pImageInfo->format || device->GetOptions()->disableDirectImageMapping)
       return { D3D11_COMMON_TEXTURE_MAP_MODE_DYNAMIC, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT };
 
     // Multi-plane and depth-stencil images have a special memory layout
@@ -1084,20 +1086,34 @@ namespace dxvk {
           VkImageLayout*        pLayout,
           VkImageCreateInfo*    pInfo) {
     const Rc<DxvkImage> image = m_texture->GetImage();
+
+    if (!m_locked.load(std::memory_order_acquire)) {
+      // Need to make sure that the image cannot be relocated. This may
+      // be entered by multiple threads, which is fine since the actual
+      // work is serialized into the CS thread and only the first call
+      // will actually modify any image state.
+      Com<ID3D11Device> device;
+      m_resource->GetDevice(&device);
+
+      static_cast<D3D11Device*>(device.ptr())->LockImage(image, 0u);
+
+      m_locked.store(true, std::memory_order_release);
+    }
+
     const DxvkImageCreateInfo& info = image->info();
-    
+
     if (pHandle != nullptr)
       *pHandle = image->handle();
-    
+
     if (pLayout != nullptr)
       *pLayout = info.layout;
-    
+
     if (pInfo != nullptr) {
       // We currently don't support any extended structures
       if (pInfo->sType != VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO
        || pInfo->pNext != nullptr)
         return E_INVALIDARG;
-      
+
       pInfo->flags          = 0;
       pInfo->imageType      = info.type;
       pInfo->format         = info.format;
