@@ -10,6 +10,7 @@
 #include "dxvk_framebuffer.h"
 #include "dxvk_image.h"
 #include "dxvk_instance.h"
+#include "dxvk_latency.h"
 #include "dxvk_memory.h"
 #include "dxvk_meta_clear.h"
 #include "dxvk_objects.h"
@@ -34,9 +35,11 @@ namespace dxvk {
    * \brief Device performance hints
    */
   struct DxvkDevicePerfHints {
-    VkBool32 preferFbDepthStencilCopy : 1;
-    VkBool32 preferFbResolve          : 1;
-    VkBool32 renderPassClearFormatBug : 1;
+    VkBool32 preferFbDepthStencilCopy   : 1;
+    VkBool32 renderPassClearFormatBug   : 1;
+    VkBool32 renderPassResolveFormatBug : 1;
+    VkBool32 preferRenderPassOps        : 1;
+    VkBool32 preferPrimaryCmdBufs       : 1;
   };
   
   /**
@@ -112,8 +115,8 @@ namespace dxvk {
      * \brief Checks whether debug functionality is enabled
      * \returns \c true if debug utils are enabled
      */
-    bool isDebugEnabled() const {
-      return bool(m_instance->extensions().extDebugUtils);
+    DxvkDebugFlags debugFlags() const {
+      return m_debugFlags;
     }
 
     /**
@@ -478,6 +481,16 @@ namespace dxvk {
     void requestCompileShader(
       const Rc<DxvkShader>&         shader);
 
+    /**
+     * \brief Creates latency tracker for a presenter
+     *
+     * The specicfic implementation and parameters used
+     * depend on user configuration.
+     * \param [in] presenter Presenter instance
+     */
+    Rc<DxvkLatencyTracker> createLatencyTracker(
+      const Rc<Presenter>&            presenter);
+
     /**
      * \brief Presents a swap chain image
      * 
@@ -485,13 +498,13 @@ namespace dxvk {
      * the submission thread. The status of this operation
      * can be retrieved with \ref waitForSubmission.
      * \param [in] presenter The presenter
-     * \param [in] presenteMode Present mode
-     * \param [in] frameId Optional frame ID
+     * \param [in] tracker Latency tracker
+     * \param [in] frameId Frame ID
      * \param [out] status Present status
      */
     void presentImage(
       const Rc<Presenter>&            presenter,
-            VkPresentModeKHR          presentMode,
+      const Rc<DxvkLatencyTracker>&   tracker,
             uint64_t                  frameId,
             DxvkSubmitStatus*         status);
     
@@ -501,10 +514,14 @@ namespace dxvk {
      * Submits the given command list to the device using
      * the given set of optional synchronization primitives.
      * \param [in] commandList The command list to submit
+     * \param [in] tracker Latency tracker
+     * \param [in] frameId Frame ID
      * \param [out] status Submission feedback
      */
     void submitCommandList(
       const Rc<DxvkCommandList>&      commandList,
+      const Rc<DxvkLatencyTracker>&   tracker,
+            uint64_t                  frameId,
             DxvkSubmitStatus*         status);
 
     /**
@@ -585,6 +602,7 @@ namespace dxvk {
     Rc<DxvkAdapter>             m_adapter;
     Rc<vk::DeviceFn>            m_vkd;
 
+    DxvkDebugFlags              m_debugFlags;
     DxvkDeviceQueueSet          m_queues;
 
     DxvkDeviceFeatures          m_features;
