@@ -55,10 +55,17 @@ namespace dxvk {
     // Initial image layout
     VkImageLayout initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
 
+    // Color space to interpret image data with. This
+    // is only meaningful for swap chain back buffers.
+    VkColorSpaceKHR colorSpace = VK_COLOR_SPACE_MAX_ENUM_KHR;
+
     // Image is used by multiple contexts so it needs
     // to be in its default layout after each submission
     VkBool32 shared = VK_FALSE;
 
+    // Image is likely to have a short lifetime
+    VkBool32 transient = VK_FALSE;
+
     // Image view formats that can
     // be used with this image
     uint32_t viewFormatCount = 0;
@@ -89,6 +96,8 @@ namespace dxvk {
     // New image layout. If undefined, the
     // default layout will not be changed.
     VkImageLayout layout = VK_IMAGE_LAYOUT_UNDEFINED;
+    // Color space to interpret the image in
+    VkColorSpaceKHR colorSpace = VK_COLOR_SPACE_MAX_ENUM_KHR;
     // Number of new view formats to add
     uint32_t viewFormatCount = 0u;
     // View formats to add to the compatibility list
@@ -98,6 +107,18 @@ namespace dxvk {
   };
 
 
+  /**
+   * \brief Image properties stored in the view
+   *
+   * Used to reduce some pointer chasing.
+   */
+  struct DxvkImageViewImageProperties {
+    VkImageLayout         layout  = VK_IMAGE_LAYOUT_UNDEFINED;
+    VkSampleCountFlagBits samples = VK_SAMPLE_COUNT_FLAG_BITS_MAX_ENUM;
+    VkAccessFlags         access  = 0u;
+  };
+
+
   /**
    * \brief Virtual image view
    *
@@ -266,6 +287,32 @@ namespace dxvk {
         view->imageSubresources());
     }
 
+    /**
+     * \brief Queries the default image layout
+     *
+     * Used when binding the view as a descriptor.
+     * \returns Default image layout
+     */
+    VkImageLayout defaultLayout() const {
+      return m_properties.layout;
+    }
+
+    /**
+     * \brief Checks whether the image is multisampled
+     * \returns \c true if the image is multisampled
+     */
+    bool isMultisampled() const {
+      return m_properties.samples > VK_SAMPLE_COUNT_1_BIT;
+    }
+
+    /**
+     * \brief Checks whether the image has graphics stores
+     *
+     * This may include attachment access for render passes.
+     * \returns \c true if the image has graphics pipeline stores
+     */
+    bool hasGfxStores() const;
+
   private:
 
     DxvkImage*              m_image     = nullptr;
@@ -273,12 +320,16 @@ namespace dxvk {
 
     uint32_t                m_version   = 0u;
 
+    DxvkImageViewImageProperties m_properties = { };
+
     std::array<VkImageView, ViewCount> m_views = { };
 
     VkImageView createView(VkImageViewType type) const;
 
     void updateViews();
 
+    void updateProperties();
+
   };
 
 
@@ -571,11 +622,36 @@ namespace dxvk {
      * \brief Retrieves resource ID for barrier tracking
      * \returns Unique resource ID
      */
-    uint64_t getResourceId() const {
+    bit::uint48_t getResourceId() const {
       constexpr static size_t Align = alignof(DxvkResourceAllocation);
-      return reinterpret_cast<uintptr_t>(m_storage.ptr()) / (Align & -Align);
+      return bit::uint48_t(reinterpret_cast<uintptr_t>(m_storage.ptr()) / (Align & -Align));
     }
 
+    /**
+     * \brief Computes virtual offset of a subresource
+     *
+     * Used for hazard tracking. Ignores the aspect mask and
+     * only takes the mip level and array layer into account.
+     * \param [in] mip Mip level index
+     * \param [in] layer Array layer index
+     */
+    uint64_t getTrackingAddress(uint32_t mip, uint32_t layer) const {
+      // Put layers within the same mip into a contiguous range. This works well
+      // for not only transfer operations but also most image view use cases.
+      return uint64_t((m_info.numLayers * mip) + layer) << 48u;
+    }
+
+    /**
+     * \brief Computes virtual offset of a specific image region
+     *
+     * Used for more granular hazard tracking. This interleaves coordinate
+     * bits in order to compute a unique address for each pixel.
+     * \param [in] mip Mip level index
+     * \param [in] layer Array layer index
+     * \param [in] coord Pixel coordinate within the subresource
+     */
+    uint64_t getTrackingAddress(uint32_t mip, uint32_t layer, VkOffset3D coord) const;
+
     /**
      * \brief Creates or retrieves an image view
      *
@@ -610,6 +686,14 @@ namespace dxvk {
      */
     void setDebugName(const char* name);
 
+    /**
+     * \brief Retrieves debug name
+     * \returns Debug name
+     */
+    const char* getDebugName() const {
+      return m_debugName.c_str();
+    }
+
   private:
 
     Rc<vk::DeviceFn>            m_vkd;
@@ -721,4 +805,10 @@ namespace dxvk {
     return m_views[viewType];
   }
 
+
+  inline bool DxvkImageView::hasGfxStores() const {
+    return (m_properties.access & (VK_ACCESS_SHADER_WRITE_BIT | VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT))
+        && (m_image->hasGfxStores());
+  }
+
 }
