@@ -882,7 +882,7 @@ namespace dxvk {
             uint32_t                vector1,
             uint32_t                vector2);
     
-    uint32_t opFOrdNotEqual(
+    uint32_t opFUnordNotEqual(
             uint32_t                resultType,
             uint32_t                vector1,
             uint32_t                vector2);
@@ -1277,6 +1277,10 @@ namespace dxvk {
 
     void opEndInvocationInterlock();
 
+    uint32_t opSinCos(
+            uint32_t                x,
+            bool                    useBuiltIn);
+
   private:
     
     uint32_t m_version;
@@ -1332,6 +1336,15 @@ namespace dxvk {
             std::unordered_set<uint32_t>& reachableBlocks,
             std::unordered_set<uint32_t>& mergeBlocks);
 
+    static constexpr double sincosTaylorFactor(uint32_t power) {
+      double result = 1.0;
+
+      for (uint32_t i = 1; i <= power; i++)
+        result *= pi * 0.25f / double(i);
+
+      return result;
+    }
+
   };
   
-}
\ No newline at end of file
+}
